/*
 * This file is part of vospace-file-catalog
 * Copyright (C) 2021 Istituto Nazionale di Astrofisica
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

-- Generates the path from parent_path and id
CREATE OR REPLACE FUNCTION path(parent_path ltree, id bigint) RETURNS ltree AS
$func$
BEGIN
    RETURN text2ltree(CASE
        WHEN parent_path IS NULL THEN ''
        WHEN parent_path::varchar = '' THEN id::varchar
        ELSE (parent_path::varchar || '.' || id::varchar)
    END);
END
$func$ LANGUAGE plpgsql IMMUTABLE; 

-- Testing the function
SELECT CASE WHEN
path(NULL, 0) = text2ltree('')
AND path(text2ltree(''), 1) = text2ltree('1')
AND path(text2ltree('1'), 2) = text2ltree('1.2')
THEN 'PASSED' ELSE 'FAILED' END AS test;


-- Create node path indexes and generated columns using the previously defined function

CREATE INDEX file_parent_path_gist_idx ON node USING GIST(parent_path);
CREATE INDEX file_parent_path_idx ON node USING btree(parent_path);

ALTER TABLE node ADD COLUMN path ltree GENERATED ALWAYS AS (path(parent_path, node_id)) STORED;
CREATE INDEX file_path_gist_idx ON node USING GIST(path);
CREATE UNIQUE INDEX file_path_idx ON node USING btree(path);

ALTER TABLE node ADD COLUMN relative_path ltree GENERATED ALWAYS AS (path(parent_relative_path, node_id)) STORED;
CREATE INDEX file_rel_path_gist_idx ON node USING GIST(relative_path);

-- Create root node
INSERT INTO node (parent_path, parent_relative_path, name, type, creator_id, is_public) VALUES (NULL, NULL, '', 'container', '0', true);
